# Sage version information for shell scripts.
#
# #31049: The following line is valid shell code but not valid Python code,
# which stops "setup.py develop" from rewriting it as a Python file.
:
# This file is auto-generated by the sage-update-version script, do not edit!
SAGE_VERSION='10.5.rc1'
SAGE_RELEASE_DATE='2024-11-23'
SAGE_VERSION_BANNER='SageMath version 10.5.rc1, Release Date: 2024-11-23'
